library threshold;

import 'dart:convert';

import 'package:archive/archive.dart';
import 'package:lzstring/lzstring.dart';

String compressLzstring({String? compress, String? decompress}) =>
    decompress != null
        ? LZString.decompressFromBase64Sync(decompress)!
        : "!${LZString.compressToBase64Sync(compress ?? "error")!}";

String compressBzip2({String? compress, String? decompress}) => decompress !=
        null
    ? String.fromCharCodes(BZip2Decoder().decodeBytes(base64Decode(decompress)))
    : "@${base64Encode(BZip2Encoder().encode((compress ?? "error").codeUnits))}";

String compressGzip({String? compress, String? decompress}) => decompress !=
        null
    ? String.fromCharCodes(GZipDecoder().decodeBytes(base64Decode(decompress)))
    : "*${base64Encode(GZipEncoder().encode((compress ?? "error").codeUnits, level: 9)!)}";

String compressZLib({String? compress, String? decompress}) => decompress !=
        null
    ? String.fromCharCodes(
        const ZLibDecoder().decodeBytes(base64Decode(decompress)))
    : "^${base64Encode(const ZLibEncoder().encode((compress ?? "error").codeUnits, level: 9))}";

String debugCompress(String data) {
  Map<String, String?> all = {
    "RAW": data,
    "LZS": _attempt(() => compressLzstring(compress: data)),
    "BZ2": _attempt(() => compressBzip2(compress: data)),
    "GZI": _attempt(() => compressGzip(compress: data)),
    "ZLI": _attempt(() => compressZLib(compress: data)),
  };

  for (String key in all.keys) {
    print("$key: ${all[key]?.length ?? "-1"}: ${all[key] ?? "ERRORED"}");
  }

  print("   ");

  return compress(data);
}

String? _attempt(String Function() f) {
  try {
    return f();
  } catch (ignored) {
    return null;
  }
}

String compress(String data) =>
    [
      data,
      _attempt(() => compressLzstring(compress: data)),
      _attempt(() => compressBzip2(compress: data)),
      _attempt(() => compressGzip(compress: data)),
      _attempt(() => compressZLib(compress: data)),
    ]
        .where((element) => element != null)
        .reduce((a, b) => a!.length < b!.length ? a : b) ??
    "ERROR";

String decompress(String data) {
  if (data.substring(0, 1) == "!") {
    return compressLzstring(decompress: data.substring(1));
  } else if (data.substring(0, 1) == "@") {
    return compressBzip2(decompress: data.substring(1));
  } else if (data.substring(0, 1) == "*") {
    return compressGzip(decompress: data.substring(1));
  } else if (data.substring(0, 1) == "^") {
    return compressZLib(decompress: data.substring(1));
  } else {
    return data;
  }
}
